<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\DataLayer\Builder;

use Flat101\EnhancedEcommerce\Model\DataLayer\Dto;
use Flat101\EnhancedEcommerce\Model\Handler;
use Flat101\EnhancedEcommerce\Helper\Config;
use Flat101\EnhancedEcommerce\Model\UnsetProperty;
use Magento\Catalog\Api\CategoryRepositoryInterface;
use Magento\Catalog\Model\Product as ProductEntity;
use Magento\ConfigurableProduct\Model\Product\Type\Configurable;
use Magento\Framework\Pricing\Helper\Data as PricingHelper;

class ImpressionBuilder
{
    use UnsetProperty;

    /** @var Handler\ProductHandler */
    private $productHandler;

    /** @var CategoryRepositoryInterface */
    private $categoryRepository;

    /** @var Config */
    private $config;

    /** @var Configurable */
    private $configurableProduct;

    /** @var PricingHelper */
    private $pricingHelper;

    public function __construct(
        Handler\ProductHandler $productHandler,
        CategoryRepositoryInterface $categoryRepository,
        Config $config,
        Configurable $configurableProduct,
        PricingHelper $pricingHelper
    ) {
        $this->productHandler = $productHandler;
        $this->categoryRepository = $categoryRepository;
        $this->config = $config;
        $this->configurableProduct = $configurableProduct;
        $this->pricingHelper = $pricingHelper;
    }

    public function build(ProductEntity $entity, string $list, $additionalInfo = null): Dto\Impression\ImpressionProduct
    {
        $this->productHandler->setProduct($entity);

        $category = $entity->getCategory();
        if (null === $category) {
            $categoryIds = $entity->getCategoryIds();
            $category = $this->categoryRepository->get(reset($categoryIds));
        }

        $this->productHandler->setCategory($category);

        $productImpressionDto = new Dto\Impression\ImpressionProduct();
        $productImpressionDto->id = $entity->getData($this->productHandler->productIdentifier());
        $productImpressionDto->name = $entity->getName();
        $productImpressionDto->price = $this->getPrice($entity);
        $productImpressionDto->category = $additionalInfo['category'] ?? $this->productHandler->getCategoryPath(true);

        if ($brandValue = $this->productHandler->getBrandValue()) {
            $productImpressionDto->brand = $brandValue;
        }

        if (!is_null($variantValue = $this->checkVariantValue($entity))) {
            $productImpressionDto->variant = $variantValue;
        }

        $productImpressionDto->position = $additionalInfo['position'] ?? '0';

        if (!is_null($list)) {
            $productImpressionDto->list = $list;
        } else {
            $this->unset($productImpressionDto, ['list']);
        }

        return $productImpressionDto;
    }

    private function getPrice(ProductEntity $product): float
    {
        if ($product->getTypeId() === Configurable::TYPE_CODE) {
            return floatval($this->getConfigurablePrice($product));
        }
        return floatval($product->getSpecialPrice()) ?: floatval($product->getData('price'));
    }

    private function getConfigurablePrice(ProductEntity $product)
    {
        $childProductPrice = [];
        $childProducts = $this->configurableProduct->getUsedProducts($product);

        foreach($childProducts as $child) {
            $price = number_format(floatval($child->getPrice()), 2, '.', '');
            $finalPrice = number_format(floatval($child->getFinalPrice()), 2, '.', '');

            if($price == $finalPrice) {
                $childProductPrice[] = $price;
            } else if($finalPrice < $price) {
                $childProductPrice[] = $finalPrice;
            }
        }

        return min($childProductPrice);
    }

    private function checkVariantValue($item): ?string
    {
        return $this->productHandler->checkVariantValue($item);
    }
}
