<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\Handler;

use Flat101\Base\Service\GetCurrentCategoryService as CategoryRegistry;
use Flat101\Base\Service\GetCurrentProductService as ProductRegistry;
use Flat101\EnhancedEcommerce\Helper\Config;
use Flat101\EnhancedEcommerce\Model\Config\Source\CategoryFormat;
use Flat101\EnhancedEcommerce\Model\Config\Source\VariantUse;
use Magento\Catalog\Api\Data\CategoryInterface;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Helper\Data as CatalogHelper;
use Magento\Catalog\Model\Category;
use Magento\Catalog\Model\Product as ProductCatalog;
use Magento\Catalog\Model\ResourceModel\Category\Collection;
use Magento\Framework\App\Request\Http;

class ProductHandler
{
    private const DEFAULT_PRODUCT_POSITION = 1;

    /** @var CategoryRegistry */
    protected $categoryRegistry;

    /** @var ProductRegistry */
    protected $productRegistry;

    /** @var CatalogHelper */
    protected $catalogHelper;

    /** @var Config */
    protected $config;

    /** @var null|ProductCatalog */
    protected $product;

    /** @var null|Category */
    protected $category;

    /** @var ItemHandler */
    private $itemHandler;

    /** @var Http */
    private $request;

    public function __construct(
        CategoryRegistry $categoryRegistry,
        ProductRegistry $productRegistry,
        CatalogHelper $catalogHelper,
        Config $config,
        ItemHandler $itemHandler,
        Http $request
    ) {
        $this->categoryRegistry = $categoryRegistry;
        $this->productRegistry = $productRegistry;
        $this->catalogHelper = $catalogHelper;
        $this->config = $config;
        $this->itemHandler = $itemHandler;
        $this->request = $request;
    }

    public function getCategoryName(): ?string
    {
        $category = $this->getCategory();

        return null !== $category ? $category->getName() : null;
    }

    public function getCategoryPath($impressions = false): ?string
    {
        try {
            $categoriesArray = $this->getBreadCrumbPath();

            if (!is_array($categoriesArray))
                return null;

            // For product => Remove last item (product name)
            if (!$impressions)
                array_pop($categoriesArray);

            if (count($categoriesArray) == 0 && $product = $this->getProduct()) {
                $category = $product->getCategoryCollection()->addAttributeToSelect('name')->getFirstItem();

                if ($category)
                    $categoriesArray[] = $category->getName();
            }

            if (count($categoriesArray) > 1 && $this->config->getCategoryFormat() == CategoryFormat::ONLY_MAIN_FORMAT) {
                $arrayReversed = array_reverse($categoriesArray);
                return array_pop($arrayReversed);
            }

            $categoryName = implode(' ' . $this->config->getCategorySeparator() . ' ', $categoriesArray);
        } catch (\Exception $e) {
            $categoryName = '';
        }

        return $categoryName;
    }

    protected function getBreadCrumbPath(): ?array
    {
        $breadCrumbs = $this->catalogHelper->getBreadcrumbPath();

        // Amasty Brands: Add impressions info
        $handle = $this->request->getFullActionName();
        if ($handle === 'ambrand_index_index') {
            $lastPathInfo = array_slice(explode('/', rtrim($this->request->getPathInfo(), '/')), -1);
            $breadCrumbs[] = ucwords(array_pop($lastPathInfo));
            return $breadCrumbs;
        }

        if (count($breadCrumbs) == 0) {
            return null;
        }

        return array_column($breadCrumbs, 'label');
    }

    public function getProduct(): ?ProductInterface
    {
        if (null !== $this->product) {
            return $this->product;
        }

        return $this->productRegistry->getProduct();
    }

    public function setProduct(ProductInterface $product): ProductInterface
    {
        $this->product = $product;

        return $this->product;
    }

    public function getCategory(): ?CategoryInterface
    {
        if (null !== $this->category) {
            return $this->category;
        }

        return $this->categoryRegistry->getCategory();
    }

    public function setCategory(Category $category): Category
    {
        $this->category = $category;

        return $this->category;
    }

    public function getBrandValue(): ?string
    {
        $brand = null;

        $attributeCode = $this->config->getBrandAttribute();
        if (null !== $attributeCode) {
            $attributes = $this->getProduct()->getAttributes();
            $brand = isset($attributes[$attributeCode])
                ? $this->getProduct()->getAttributeText($attributeCode)
                : $this->getProduct()->getData($attributeCode);

            if (!$brand) {
                $customAttribute = $this->getProduct()->getCustomAttribute($attributeCode);

                if (null !== $customAttribute) {
                    $brand = $customAttribute->getValue();
                }
            }

            if (\is_array($brand)) {
                $brand = implode(',', $brand);
            }
        }

        return $brand ?: null;
    }

    public function getCategoriesPath(): string
    {
        /** @var Collection $collection */
        $collection = $this->getProduct()->getCategoryCollection()
            ->addAttributeToSelect('name');

        $categories = [];
        /** @var Category $item */
        foreach ($collection as $item) {
            $categories[] = $item->getName();
        }

        return \implode('/', $categories);
    }

    public function productIdentifier(): string
    {
        return $this->config->getProductIdentifier();
    }

    public function getVariantValue($item): ?string
    {
        $finalPrice = $item->getPriceInfo()->getPrice('final_price')->getValue();
        $oldPrice = $item->getPriceInfo()->getPrice('regular_price')->getValue();

        if ($finalPrice < $oldPrice)  {
            return (string)round(100 - (($finalPrice * 100) / $oldPrice)) . '%';
        }

        return null;
    }

    public function checkVariantValue($item): ?string
    {
        // Default variant Use
        if ($this->config->getVariantUse() == VariantUse::DEFAULT_USAGE && null !== $item) {
            return $this->itemHandler->getVariant($item);
        }

        // Discount variant Use
        if ($this->config->getVariantUse() == VariantUse::DISCOUNT_USAGE) {
            return $this->getVariantValue($item);
        }

        return null;
    }
}
